import 'package:diameter/components/detail.dart';
import 'package:diameter/components/dialogs.dart';
import 'package:diameter/config.dart';
import 'package:diameter/navigation.dart';
import 'package:flutter/material.dart';
import 'package:diameter/components/forms.dart';
import 'package:diameter/models/accuracy.dart';

class AccuracyDetailScreen extends StatefulWidget {
  static const String routeName = '/accuracy';
  final int id;

  const AccuracyDetailScreen({Key? key, this.id = 0}) : super(key: key);

  @override
  _AccuracyDetailScreenState createState() => _AccuracyDetailScreenState();
}

class _AccuracyDetailScreenState extends State<AccuracyDetailScreen> {
  Accuracy? _accuracy;
  bool _isNew = true;
  bool _isSaving = false;

  final GlobalKey<FormState> _accuracyForm = GlobalKey<FormState>();

  final _valueController = TextEditingController(text: '');
  final _confidenceRatingController = TextEditingController(text: '');
  final _notesController = TextEditingController(text: '');
  bool _forCarbsRatio = false;
  bool _forPortionSize = false;

  @override
  void initState() {
    super.initState();
    reload();
    if (_accuracy != null) {
      _valueController.text = _accuracy!.value;
      _forCarbsRatio = _accuracy!.forCarbsRatio;
      _forPortionSize = _accuracy!.forPortionSize;
      _confidenceRatingController.text =
          (_accuracy!.confidenceRating ?? '').toString();
      _notesController.text = _accuracy!.notes ?? '';
    }
  }

  void reload() {
    if (widget.id != 0) {
      setState(() {
        _accuracy = Accuracy.get(widget.id);
      });
    }
    _isNew = _accuracy == null;
  }

  void handleSaveAction() async {
    setState(() {
      _isSaving = true;
    });
    if (_accuracyForm.currentState!.validate()) {
      Accuracy.box.put(Accuracy(
        id: widget.id,
        value: _valueController.text,
        forCarbsRatio: _forCarbsRatio,
        forPortionSize: _forPortionSize,
        confidenceRating: int.tryParse(_confidenceRatingController.text),
        notes: _notesController.text,
      ));
      Navigator.pop(context, '${_isNew ? 'New' : ''} Accuracy saved');
    }
    setState(() {
      _isSaving = false;
    });
  }

  void handleCancelAction() {
    if (showConfirmationDialogOnCancel &&
      (_isNew &&
        (_forCarbsRatio ||
            _forPortionSize ||
            _valueController.text != '' ||
            int.tryParse(_confidenceRatingController.text) != null ||
            _notesController.text != '')) ||
  (!_isNew &&
      (_forCarbsRatio != _accuracy!.forCarbsRatio ||
          _forPortionSize != _accuracy!.forPortionSize ||
          _accuracy!.value != _valueController.text ||
          int.tryParse(_confidenceRatingController.text) !=
              _accuracy!.confidenceRating ||
          (_accuracy!.notes ?? '') != _notesController.text))) {
      Dialogs.showCancelConfirmationDialog(
        context: context,
        isNew: _isNew,
        onSave: handleSaveAction,
      );
    } else {
      Navigator.pop(context);
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text(_isNew ? 'New Accuracy' : _accuracy!.value),
      ),
      drawer: const Navigation(currentLocation: AccuracyDetailScreen.routeName),
      body: SingleChildScrollView(
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.stretch,
          children: <Widget>[
            StyledForm(
              formState: _accuracyForm,
              fields: [
                TextFormField(
                  controller: _valueController,
                  decoration: const InputDecoration(
                    labelText: 'Name',
                  ),
                  validator: (value) {
                    if (value!.trim().isEmpty) {
                      return 'Empty name';
                    }
                    return null;
                  },
                ),
                StyledBooleanFormField(
                  value: _forCarbsRatio,
                  label: 'for carbs ratio',
                  onChanged: (value) {
                    setState(() {
                      _forCarbsRatio = value;
                    });
                  },
                ),
                StyledBooleanFormField(
                  value: _forPortionSize,
                  label: 'for portion size',
                  onChanged: (value) {
                    setState(() {
                      _forPortionSize = value;
                    });
                  },
                ),
                TextFormField(
                  controller: _confidenceRatingController,
                  keyboardType: TextInputType.number,
                  decoration: const InputDecoration(
                    labelText: 'Confidence Rating',
                  ),
                ),
                TextFormField(
                  controller: _notesController,
                  keyboardType: TextInputType.multiline,
                  decoration: const InputDecoration(
                    labelText: 'Notes',
                    alignLabelWithHint: true,
                  ),
                ),
              ],
            ),
          ],
        ),
      ),
      bottomNavigationBar: DetailBottomRow(
        onCancel: handleCancelAction,
        onSave: _isSaving ? null : handleSaveAction,
      ),
    );
  }
}
