import 'package:diameter/components/dialogs.dart';
import 'package:diameter/config.dart';
import 'package:diameter/models/log_entry.dart';
import 'package:diameter/models/log_event.dart';
// import 'package:diameter/models/log_event_type.dart';
import 'package:diameter/screens/log/log_event_detail.dart';
import 'package:diameter/utils/date_time_utils.dart';
import 'package:flutter/material.dart';
// import 'package:diameter/components/progress_indicator.dart';

class ActiveLogEventListScreen extends StatefulWidget {
  static const String routeName = '/active-log-events';

  final LogEntry? endLogEntry;
  final Function()? onSetEndTime;

  const ActiveLogEventListScreen(
      {Key? key, this.endLogEntry, this.onSetEndTime})
      : super(key: key);

  @override
  _ActiveLogEventListScreenState createState() =>
      _ActiveLogEventListScreenState();
}

class _ActiveLogEventListScreenState extends State<ActiveLogEventListScreen> {
  List<LogEvent> _activeLogEvents = [];

  void refresh({String? message}) {
    setState(() {
      _activeLogEvents = LogEvent.getAllOngoing();
    });

    setState(() {
      if (message != null) {
        var snackBar = SnackBar(
          content: Text(message),
          duration: const Duration(seconds: 2),
        );
        ScaffoldMessenger.of(context)
          ..removeCurrentSnackBar()
          ..showSnackBar(snackBar);
      }
    });
  }

  void onStop(LogEvent event) async {
    event.endTime = DateTime.now();
    event.endLogEntry.target =
        widget.endLogEntry ?? LogEntry(time: DateTime.now());
    LogEvent.put(event);
    refresh();
    if (widget.onSetEndTime != null) {
      widget.onSetEndTime!();
    }
  }

  void handleStopAction(LogEvent event) async {
    if (showConfirmationDialogOnStopEvent) {
      Dialogs.showConfirmationDialog(
        context: context,
        onConfirm: () => onStop(event),
        message: 'Are you sure you want to end this Event?',
      );
    } else {
      onStop(event);
    }
  }

  void onDelete(LogEvent event) {
    LogEvent.remove(event.id);
    refresh(message: 'Event deleted');
  }

  void handleDeleteAction(LogEvent event) async {
    if (showConfirmationDialogOnDelete) {
      Dialogs.showConfirmationDialog(
        context: context,
        onConfirm: () => onDelete(event),
        message: 'Are you sure you want to delete this Event?',
      );
    } else {
      onDelete(event);
    }
  }

  @override
  void initState() {
    super.initState();
    refresh();
  }

  @override
  Widget build(BuildContext context) {
    return SingleChildScrollView(
      padding: const EdgeInsets.only(top: 10.0),
      child: Column(
        children: [
          // TODO: make action button instead of appbar
          AppBar(
            title: const Text('Active Events'),
            primary: false,
            automaticallyImplyLeading: false,
            actions: [
              IconButton(
                icon: const Icon(Icons.add),
                onPressed: () {
                  Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (context) => LogEventDetailScreen(
                        logEntry: widget.endLogEntry!,
                      ),
                    ),
                  ).then((message) => refresh(message: message));
                },
              ),
              IconButton(icon: const Icon(Icons.refresh), onPressed: refresh),
            ],
          ),
          _activeLogEvents.isNotEmpty ?
          ListView.builder(
            shrinkWrap: true,
            itemCount: _activeLogEvents.length,
            itemBuilder: (context, index) {
              final event = _activeLogEvents[index];
              return ListTile(
                title: Row(
                  mainAxisSize: MainAxisSize.max,
                  children: [
                    Expanded(
                      child: Text(event.eventType.target?.value ?? ''),
                    ),
                  ],
                ),
                subtitle: Text(
                    '${DateTimeUtils.displayDateTime(event.time)}${event.hasEndTime ? ' - ${DateTimeUtils.displayDateTime(event.endTime)}' : ''}'),
                trailing: Row(
                  mainAxisSize: MainAxisSize.min,
                  children: [
                    IconButton(
                      icon: const Icon(
                        Icons.delete,
                        color: Colors.blue,
                      ),
                      onPressed: () => handleStopAction(event),
                    ),
                    IconButton(
                      icon: const Icon(
                        Icons.delete,
                        color: Colors.blue,
                      ),
                      onPressed: () => handleDeleteAction(event),
                    ),
                  ],
                ),
              );
            },
          ) : const Center(
              child: Text('There are no currently ongoing events!'),
            ),
        ],
      ),
    );
  }
}
