import 'package:diameter/components/dialogs.dart';
import 'package:diameter/config.dart';
import 'package:diameter/models/log_entry.dart';
import 'package:diameter/models/log_event.dart';
import 'package:diameter/screens/log/log_event_detail.dart';
import 'package:diameter/utils/date_time_utils.dart';
import 'package:flutter/material.dart';

class LogEventListScreen extends StatefulWidget {
  final LogEntry logEntry;
  final Function() reload;

  const LogEventListScreen({Key? key, required this.logEntry, required this.reload})
      : super(key: key);

  @override
  _LogEventListScreenState createState() => _LogEventListScreenState();
}

class _LogEventListScreenState extends State<LogEventListScreen> {
  void reload({String? message}) {
    widget.reload();

    setState(() {
      if (message != null) {
        var snackBar = SnackBar(
          content: Text(message),
          duration: const Duration(seconds: 2),
        );
        ScaffoldMessenger.of(context)
          ..removeCurrentSnackBar()
          ..showSnackBar(snackBar);
      }
    });
  }

  void handleEditAction(LogEvent event) {
    Navigator.push(
      context,
      MaterialPageRoute(
        builder: (context) => LogEventDetailScreen(
          endLogEntry: widget.logEntry,
          logEvent: event,
        ),
      ),
    ).then((message) => reload(message: message));
  }

  void onDelete(LogEvent logEvent) {
    LogEvent.remove(logEvent.id);
    reload(message: 'Event deleted');
  }

  void handleDeleteAction(LogEvent logEvent) async {
    if (showConfirmationDialogOnDelete) {
      Dialogs.showConfirmationDialog(
        context: context,
        onConfirm: () => onDelete(logEvent),
        message: 'Are you sure you want to delete this Event?',
      );
    } else {
      onDelete(logEvent);
    }
  }

  @override
  Widget build(BuildContext context) {
    return Column(
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          // TODO: add button for active events
          Expanded(
            child: (widget.logEntry.events.isNotEmpty || widget.logEntry.endedEvents.isNotEmpty)
              ? ListView.builder(
                  shrinkWrap: true,
                  itemCount: widget.logEntry.events.length + widget.logEntry.endedEvents.length,
                  itemBuilder: (context, index) {
                    final event = (widget.logEntry.events + widget.logEntry.endedEvents)[index];
                    return ListTile(
                      onTap: () {
                        handleEditAction(event);
                      },
                      title: Row(
                        mainAxisSize: MainAxisSize.max,
                        children: [
                          Expanded(
                              child: Text(event.eventType.target?.value ?? '')),
                        ],
                      ),
                      subtitle: Text(
                          '${DateTimeUtils.displayDateTime(event.time)}${event.hasEndTime ? ' - ${DateTimeUtils.displayDateTime(event.endTime, fallback: '(ongoing)')}' : ''}'),
                      trailing: Row(
                        mainAxisSize: MainAxisSize.min,
                        children: [
                          IconButton(
                            icon: const Icon(
                              Icons.delete,
                              color: Colors.blue,
                            ),
                            onPressed: () => handleDeleteAction(event),
                          ),
                        ],
                      ),
                    );
                  })
              : const Center(
                child: Text('You have not added any Events to this Log Entry yet!'),
              ),
          ),
        ],
    );
  }
}
